@@ -1,4 +1,3 @@
-
 /***************************************************************************
  *   copyright           : (C) 2002 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -13,6 +12,12 @@
 #ifndef CHARSETS_H
 #define CHARSETS_H
 
+/* You MUST call this function before all other functions.
+ * If charset is NULL, locale specific charset will be returned
+ *   on get_syste_charset().
+ */
+void charset_init (char* charset);
+
 char* get_system_charset ();
 
 enum repmode {
@@ -50,9 +55,11 @@ int ucs4cmp (const ucs4char_t* s1, const ucs4char_t* s2);
 ucs4char_t* ucs4str (const ucs4char_t* haystack, const ucs4char_t* needle);
 
 //these two are mainly used by the other function from below
-ucs4char_t* convert_to_internal (char* from_code, char* input,
+ucs4char_t* convert_to_internal (const char* from_code,
+				 char* input,
 				 size_t insize);
-char* convert_from_internal (char* to_code, ucs4char_t* input,
+char* convert_from_internal (const char* to_code,
+			     ucs4char_t* input,
 			     enum repmode replacement_mode);
 
 //convert from/to the system's charset
@@ -70,4 +77,9 @@ ucs4char_t* convert_from_ucs2 (ucs2char_t* input);
 char* convert_to_ucs2_hexstring (ucs4char_t* input);
 ucs4char_t* convert_from_ucs2_hexstring (char* input);
 
+/* count how many GSM-encoded UCS-4 characters fit into
+ * a max of count bytes
+ */
+unsigned int gsm_count (ucs4char_t* input, unsigned int count);
+
 #endif
