@@ -12,20 +12,14 @@
 //own headers
 #include <charsets.h>
 #include <helper.h>
+#include "ucs2.h"
 
 //standard headers
 #include <stdio.h>
 #include <string.h>
 
-#include <config.h>
-#ifdef WORDS_BIGENDIAN
-#  define UCS2_CHARSET "UCS-2BE"
-#else
-#  define UCS2_CHARSET "UCS-2LE"
-#endif
-
 ucs4char_t* convert_from_ucs2 (ucs2char_t* input) {
-  return convert_to_internal(UCS2_CHARSET,
+  return convert_to_internal(ucs2_get_iconv_charset(),
 			     (unsigned char*)input,
 			     ucs2len(input)*sizeof(ucs2char_t));
 }
