@@ -11,19 +11,13 @@
 
 #include <charsets.h>
 #include <helper.h>
+#include "ucs2.h"
 
 #include <stdio.h>
 #include <string.h>
 
-#include <config.h>
-#ifdef WORDS_BIGENDIAN
-#  define UCS2_CHARSET "UCS-2BE"
-#else
-#  define UCS2_CHARSET "UCS-2LE"
-#endif
-
 ucs2char_t* convert_to_ucs2 (ucs4char_t* input) {
-  return (ucs2char_t*)convert_from_internal(UCS2_CHARSET,
+  return (ucs2char_t*)convert_from_internal(ucs2_get_iconv_charset(),
 					    input,
 					    REPMODE_ESCAPE_CHARS);
 }
