@@ -14,6 +14,59 @@
 
 #include <string.h>
 
+struct args_def args_list[] = {
+  {"remove",'r',ARGS_PARAM_NONE},
+  {"send",'s',ARGS_PARAM_NONE},
+  {"get",'g',ARGS_PARAM_NONE},
+
+  {"bitmap",'B',ARGS_PARAM_OPT}, //deprecated
+  {"midi",'M',ARGS_PARAM_OPT}, //deprecated
+  {"vcal",'C',ARGS_PARAM_OPT}, //deprecated
+  {"vcard",'F',ARGS_PARAM_OPT}, //deprecated
+
+  {"binary",'N',ARGS_PARAM_NONE},
+  {"pbook",'P',ARGS_PARAM_NONE},
+  {"sms",'S',ARGS_PARAM_OPT}, //parameter is deprecated
+
+  {"info",'i',ARGS_PARAM_NONE},
+  {"mem-info",0,ARGS_PARAM_NONE},
+  {"lock-info",0,ARGS_PARAM_NONE},
+  {"set-time",0,ARGS_PARAM_NONE},
+  {"set-smsc",0,ARGS_PARAM_NONE},
+  {"toggle-lock",0,ARGS_PARAM_REQ}, //since 0.7.4
+  {"dial",0,ARGS_PARAM_NONE}, //since 0.7.4
+  {"reset",0,ARGS_PARAM_NONE},
+
+  {"mem",0,ARGS_PARAM_REQ},
+  {"slot",0,ARGS_PARAM_REQ},
+  {"smsmem",0,ARGS_PARAM_REQ}, //deprecated
+  {"text",'t',ARGS_PARAM_REQ},
+  {"unicode",0,ARGS_PARAM_NONE},
+  {"number",'n',ARGS_PARAM_REQ},
+  {"flash",0,ARGS_PARAM_NONE},
+  {"srr",0,ARGS_PARAM_NONE},
+  {"direct",0,ARGS_PARAM_NONE},
+  {"sort",0,ARGS_PARAM_REQ}, //since 0.7.3
+  {"system-charset",0,ARGS_PARAM_REQ}, //since 0.7.5
+  {"pin",0,ARGS_PARAM_REQ},
+
+  {"device",'d',ARGS_PARAM_REQ},
+  {"baud",'b',ARGS_PARAM_REQ},
+  {"device-timeout",0,ARGS_PARAM_REQ},
+  {"ignore-serial-bits",0,ARGS_PARAM_NONE},
+  {"keep-serial-bits",0,ARGS_PARAM_NONE},
+  {"start-delay",0,ARGS_PARAM_REQ},
+
+  {"out",'o',ARGS_PARAM_REQ},
+  {"pipe",'p',ARGS_PARAM_REQ},
+
+  {"verbose",'v',ARGS_PARAM_NONE},
+  {"quiet",'q',ARGS_PARAM_NONE},
+  {"help",'h',ARGS_PARAM_NONE},
+  {"version",0,ARGS_PARAM_NONE},
+  ARGS_DEF_LAST
+};
+
 void args_init (struct args_t* args) {
   args->port.device = NULL;
   args->port.baud = NULL;
@@ -40,14 +93,18 @@ void args_init (struct args_t* args) {
 
   args->info = 0;
   args->meminfo = 0;
+  args->lockinfo = 0;
   args->time = 0;
+  args->lock = NULL;
   args->smsc = 0;
+  args->dial = 0;
   args->reset = 0;
   args->help = 0;
   args->version = 0;
 
   args->verbose = 0;
   args->pin = NULL;
+  args->system_charset = NULL;
 }
 
 #include "helper.h"
@@ -70,6 +127,8 @@ void arg_fill_slot (char* argument, struct args_t* args) {
       args->myparams.slot = SCMXX_SLOT_SMS_READ;
     } else if(!strcmp(argument,"unread")) {
       args->myparams.slot = SCMXX_SLOT_SMS_UNREAD;
+    } else if (!strcmp(argument,"last")) {
+      args->myparams.slot = SCMXX_SLOT_LAST;
     } else {
       args->myparams.slot = SCMXX_SLOT_UNDEFINED;
     }
@@ -88,14 +147,11 @@ void arg_fill (int short_opt, const char* long_opt,
     args->port.baud = argument;
     break;
   case 't':
-    if (str_len(argument)) {
-      args->myparams.text=argument;
-    } else {
-      errexit("%s","Zero length text makes no sense.\n");
-    }
+    if (str_len(argument)) args->myparams.text=argument;
     break;
   case 'n':
-    args->myparams.number=argument;
+    if (str_len(argument)) args->myparams.number=argument;
+    break;
   case 'o':
     args->myFILE=argument;
     break;
@@ -198,6 +254,10 @@ void arg_fill (int short_opt, const char* long_opt,
 	args->time=1;
       } else if (!strcmp(long_opt,"set-smsc")) {
 	args->smsc=1;
+      } else if (!strcmp(long_opt,"toggle-lock")) {
+	args->lock=argument;
+      } else if (!strcmp(long_opt,"dial")) {
+	args->dial=1;
       } else if (!strcmp(long_opt,"smsmem")) { //old, compat
 	args->myparams.mem = argument;
       } else if (!strcmp(long_opt,"mem")) {
@@ -206,8 +266,12 @@ void arg_fill (int short_opt, const char* long_opt,
 	arg_fill_slot(argument,args);
       } else if (!strcmp(long_opt,"mem-info")) {
 	args->meminfo = 1;
+      } else if (!strcmp(long_opt,"lock-info")) {
+	args->lockinfo = 1;
       } else if (!strcmp(long_opt,"sort")) { //old, compat
 	args->mysmsopts.sort_mode = argument;
+      } else if (!strcmp(long_opt,"system-charset")) {
+	args->system_charset = argument;
       }
     }
     break;
