@@ -1,3 +1,4 @@
+
 /***************************************************************************
  *   copyright           : (C) 2003 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -20,6 +21,7 @@
 #define SCMXX_FTYPE_PBOOK  0x2
 #define SCMXX_FTYPE_SMS    0x4
 
+#define SCMXX_SLOT_LAST       -7
 #define SCMXX_SLOT_SMS_SENT   -6
 #define SCMXX_SLOT_SMS_UNSENT -5
 #define SCMXX_SLOT_SMS_READ   -4
@@ -75,16 +77,46 @@ struct args_t {
 
   int info;
   int meminfo;
+  int lockinfo;
   int time;
+  char* lock;
   int smsc;
+  int dial;
   int reset;
   int help;
   int version;
 
   int verbose;
   char* pin;
+  char* system_charset;
 };
 
+enum args_param_type {
+  ARGS_PARAM_NONE = 0, //no parameter allowed
+  ARGS_PARAM_OPT, //parameter is optional, does not work with libpopt
+  ARGS_PARAM_REQ //parameter is required
+};
+struct args_def {
+  char* long_name; //long option name
+  char short_name; //short option name, 0 for none
+  enum args_param_type ptype; //parameter type
+};
+#define ARGS_DEF_LAST { NULL, 0, ARGS_PARAM_NONE }
+extern struct args_def args_list[];
+
+/* return full path of filename with
+ * the configuration directory as base
+ * return value needs to be freed
+ */
+char* configfile_get_path (char* filename);
+
+/* reads a config file located at filename
+ * the config file must contain long-options only
+ * seperator of option and arguments is '='
+ * comments have a '#' at beginning of line
+ */
+char** configfile_read (char* filename);
+
 /* a function to init the variables in the struct
  */
 void args_init (struct args_t* args);
