@@ -1,4 +1,3 @@
-
 /***************************************************************************
  *   copyright           : (C) 2003 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -14,6 +13,8 @@
 #include "helper.h"
 
 #include <stdlib.h>
+#include <ctype.h>
+#include <string.h>
 
 #include "config.h"
 #ifdef HAVE_GETOPT_H
@@ -26,65 +27,43 @@ char** args_parse (int argc, char** argv, struct args_t* args) {
   int arg = -1;
   int option_index = 0;
   char** retval;
-  int i;
-
-  struct option myoptions[]={
-    {"remove",0,0,'r'},
-    {"send",0,0,'s'},
-    {"get",0,0,'g'},
-    
-    {"bitmap",2,0,'B'}, //old, compat
-    {"midi",2,0,'M'}, //old, compat
-    {"vcal",2,0,'C'}, //old, compat
-    {"vcard",2,0,'F'}, //old, compat
-    {"binary",0,0,'N'},
-    {"pbook",2,0,'P'},
-    {"sms",2,0,'S'},
-    
-    {"info",0,0,'i'},
-    {"mem-info",0,0,0},
-    {"set-time",0,0,0},
-    {"set-smsc",0,0,0},
-    {"reset",0,0,0},
-
-    {"mem",1,0,0},
-    {"slot",1,0,0},
-    {"smsmem",1,0,0}, //old, compat
-    {"text",1,0,'t'},
-    {"unicode",0,0,0},
-    {"number",1,0,'n'},
-    {"flash",0,0,0},
-    {"srr",0,0,0},
-    {"direct",0,0,0},
-    {"sort",1,0,0}, //new since 0.7.3
-    
-    {"device",1,0,'d'},
-    {"baud",1,0,'b'},
-    {"device-timeout",1,0,0},
-    {"ignore-serial-bits",0,0,0},
-    {"keep-serial-bits",0,0,0},
-    {"start-delay",1,0,0},
-    
-    {"out",1,0,'o'},
-    {"pipe",1,0,'p'},
-    
-    {"pin",1,0,0},
-    
-    {"verbose",0,0,'v'},
-    {"quiet",0,0,'q'},
-    {"help",0,0,'h'},
-    {"version",0,0,0},
-    
-    {0,0,0,0}
-  };
-  const char myoptions_short[]="rsgB::M::C::F::NP::S::it:n:d:b:o:p:vqh";
+  int i = 0;
+  struct option* myoptions;
+  struct option listEnd = { 0, 0, 0, 0 };
+  char* myoptions_short;
+  int short_count = 0;
   char* posixly_correct;
+  int k = 0;
+
+  while(args_list[i].long_name != NULL ||
+	args_list[i].short_name != 0)
+    ++i;
+  myoptions = mem_alloc((i+1)*sizeof(*myoptions),0);
+  myoptions_short = mem_alloc((3*i*sizeof(*myoptions_short))+1,0);
+  memcpy(&args_list[i+1],&listEnd,sizeof(listEnd));  
+  for (; i >= 0; --i) {
+    myoptions[i].name = args_list[i].long_name;
+    myoptions[i].val = args_list[i].short_name;
+    myoptions[i].flag = 0;
+    switch (args_list[i].ptype) {
+    case ARGS_PARAM_NONE:  myoptions[i].has_arg = 0; break;
+    case ARGS_PARAM_OPT: myoptions[i].has_arg = 2; break;
+    case ARGS_PARAM_REQ: myoptions[i].has_arg = 1; break;
+    }
+    if (isalnum((int)args_list[i].short_name)) {
+      myoptions_short[short_count++] = args_list[i].short_name;
+      for (k = 0; k < myoptions[i].has_arg; ++k) {
+	myoptions_short[short_count++] = ':';
+      }
+    }
+  }
 
   if ((posixly_correct=getenv("POSIXLY_CORRECT")) != NULL) {
     posixly_correct = str_dup(getenv("POSIXLY_CORRECT"));
   }
   setenv("POSIXLY_CORRECT","1",1);
 
+  optind = 0;
   while ((arg = getopt_long(argc, argv,
 			    myoptions_short,
 			    myoptions,
@@ -111,5 +90,8 @@ char** args_parse (int argc, char** argv, struct args_t* args) {
   }
   retval[i]=NULL;
 
+  mem_realloc(myoptions,0);
+  mem_realloc(myoptions_short,0);
+
   return retval;
 }
