@@ -1,4 +1,3 @@
-
 /***************************************************************************
  *   copyright           : (C) 2002 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -17,6 +16,7 @@
 #include "atcommand.h"
 #include "ttyaccess.h"
 #include "pbookfile.h"
+#include "gtincl.h"
 
 #include <errno.h>
 #include <stdlib.h>
@@ -50,7 +50,7 @@ void sms_manage (int action, char** files,
   }
 
   if (action == 0) {
-    errexit ("%s","You must specify a valid action.\n");
+    errexit ("%s\n",_("You must specify a valid action."));
   }
 
   if (myparams->slot == SCMXX_SLOT_UNDEFINED) {
@@ -61,7 +61,7 @@ void sms_manage (int action, char** files,
     case SCMXX_ACTION_SEND:
       break;
     default:
-      errexit("%s","If you want to remove short messages or mix actions, you must define a slot or slot type.\n");
+      errexit("%s\n",_("If you want to remove short messages or mix actions, you must define a slot or slot type."));
       break;
     }
   }
@@ -94,7 +94,7 @@ void sms_manage (int action, char** files,
     storetype=memlist[i];
   }
   if (!sms_select_mem(storetype,&range,&current_fill)) {
-    errexit("SMS storage type \"%s\" could not be selected.\n",storetype);
+    errexit(_("SMS storage type \"%s\" could not be selected.\n"),storetype);
   }
   if (memlist != NULL) {
     i=0;
@@ -149,7 +149,7 @@ void sms_manage (int action, char** files,
       } else {
 	if (myparams->text == NULL &&
 	    (files == NULL || str_len(files[0]) == 0)) {
-	  myprintf(0,"%s","Hmm, nothing to send?\n");
+	  myprintf(0,"%s\n",_("Hmm, nothing to send?"));
 	}
 	if (myparams->text != NULL) {
 	  pdu = sms_pdu_create(NULL,myparams->text,myparams->number,mysmsopts);
@@ -193,15 +193,15 @@ void sms_delete_slot (int smsslot) {
     at_gen_sms_delete(smsslot);
     ack = at_read_line();
     if (!strcmp(ack,"OK")) {
-      myprintf(0,"SMS slot %d was deleted.\n",smsslot);
+      myprintf(0,_("SMS slot %d was deleted.\n"),smsslot);
     } else if(!strcmp(ack,"+CMS ERROR: INVALID MEMORY INDEX")) {
-      errexit("%s","This index is not available.\n");
+      errexit("%s\n",_("This slot is not available."));
     } else if(strstr(ack,"ERROR")!=NULL) {
-      errexit("%s, aborting.\n",ack);
+      errexit("%s\n",ack);
     }
     mem_realloc(ack,0);
   } else {
-    errexit("%s","You must specify a valid slot number.\n");
+    errexit("%s\n",_("You must specify a valid slot number."));
   }
 }
 
@@ -222,18 +222,18 @@ void sms_delete_all (struct sms_slot_data** slist) {
 void sms_send_slot (int smsslot, char* smsnumber) {
   char* ausgabe;
 
-  if (str_len(smsnumber) > 0) smsnumber = NULL;
+  if (str_len(smsnumber) == 0) errexit("%s: %s\n",_("Error"),_("zero-length number"));
   at_gen_sms_slot_send(smsslot,smsnumber);
   ausgabe = at_read_line();
   if (!strcmp(ausgabe,"+CMS ERROR: INVALID MEMORY INDEX")) {
-    errexit("%s","This index is not available.\n");
+    errexit("%s\n",_("This slot is not available."));
   } else if(strstr(ausgabe,"ERROR")!=NULL) {
-    errexit("%s, aborting.\n",ausgabe);
+    errexit("%s\n",ausgabe);
   }
-  myprintf(0,"The message was sent. Message reference: %s\n",ausgabe+7);
+  myprintf(0,"%s %s: %s\n",_("The message was sent."),_("Message reference"),ausgabe+7);
   mem_realloc(ausgabe,0);
   ausgabe = at_read_line();
-  myprintf(0,"The phone returned: %s\n",ausgabe);
+  myprintf(0,"%s: %s\n",_("The phone returned"),ausgabe);
   mem_realloc(ausgabe,0);
 }
 
@@ -249,32 +249,32 @@ void sms_send_pdu (char* pdu, int direct) {
     at_gen_sms_store(sms_tpdu_len(pdu));
     command = AT_GEN_SMS_STORE;
   }
-  myprintf(0,"%s","Waiting for data request...\n");
+  myprintf(0,"%s\n",_("Waiting for data request..."));
   ack = at_read_line();
   if (!strncmp(ack,"> ",2)) {
     mem_realloc(ack,0);
-    myprintf(0,"%s","Sending data...\n");
+    myprintf(0,"%s\n",_("Sending data..."));
     if (tty_write_data(pdu,strlen(pdu)) == -1) {
-      errexit("ERROR on sending data: %s\n", strerror(errno));
+      errexit(_("Error on sending data: %s\n"), strerror(errno));
     }
     ausgabe = at_read_line();
     if (strstr(ausgabe,"ERROR")!=NULL) {
-      errexit("An error occured on sending the SMS: %s\n",ausgabe);
+      errexit(_("An error occured on sending the SMS: %s\n"),ausgabe);
     }
     if (direct) {
-      myprintf(0,"The message was sent. Message reference: %s\n",ausgabe+7);
+      myprintf(0,"%s %s: %s\n",_("The message was sent."),_("Message reference"),ausgabe+7);
     } else {
-      myprintf(0,"The message was saved to SMS memory slot %s\n",ausgabe+7);
+      myprintf(0,_("The message was saved to SMS memory slot %s\n"),ausgabe+7);
     }
     mem_realloc(ausgabe,0);
     ack = at_read_line();
-    myprintf(0,"The phone returned: %s\n",ack);
+    myprintf(0,"%s: %s\n",_("The phone returned"),ack);
     mem_realloc(ack,0);
   } else {
     if (!strncmp(ack,"ERROR",5)) {
-      errexit("%s","An unknown error occured.");
+      errexit("%s\n",_("An unknown error occured."));
     } else if (!strcasecmp(ack,"+CMS ERROR: MEMORY FULL")) {
-      errexit("%s","There is no slot free to store the sms.\n");
+      errexit("%s\n",_("There is no slot free to store the sms."));
     } else {
       errexit("ERROR: %s\n",ack+12);
     }
@@ -290,10 +290,10 @@ void sms_send_pdu_all (struct sms_slot_data** slist, char* number, int direct) {
   }
   while (slist[i] != NULL) {
     if (slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
-      myprintf(0,"Trying to send SMS from slot %d...\n",slist[i]->slot);
+      myprintf(0,_("Trying to send SMS from slot %d...\n"),slist[i]->slot);
     }
     if (slist[i]->type != SMS_OUTGOING) {
-      myprintf(0,"%s","Sending is only supported with SMS of outgoing type.\n");
+      myprintf(0,"%s\n",_("Sending is only possible with SMS of outgoing type."));
     } else {
       if (str_len(number) && slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
 	sms_send_slot(slist[i]->slot, number);
@@ -331,16 +331,16 @@ char* sms_pdu_create (char* file, char* text, char* number,
 	eof_present=1;
 	break;
       } else if (read_ret==-1) {
-	errexit("%s","Error while reading from file.\n");
+	errexit("%s\n",_("Error while reading from file."));
       }
     }
     if (eof_present==0) {
-      errexit("%s","SMS text exceeds buffer size.\n");
+      errexit("%s\n",_("SMS text exceeds buffer size."));
     }
     if (str_len(smsinput)==0) {
-      errexit("%s","No SMS text found (or zero length).\n");
+      errexit("%s\n",_("No SMS text found (or zero length)."));
     } else {
-      myprintf(0,"%s has %d bytes.\n",file,str_len(smsinput));
+      myprintf(0,_("%s has %d bytes.\n"),file,str_len(smsinput));
     }
   } else {
     if (str_len(text) < sizeof(smsinput)) {
@@ -350,10 +350,10 @@ char* sms_pdu_create (char* file, char* text, char* number,
 
   //if number is not a number but text, look it up
   if (is_telephone_number(number) == 0) {
-    filename = pbook_file_get_cachename();
+    filename = configfile_get_path (PBOOK_FILE_CACHENAME);
     if (filename == NULL ||
 	file_accessible_ro(filename) == 0) {
-      errexit("%s","Error: argument to number is not a number but phonebook cache is not present.\n");
+      errexit("%s: %s\n",_("Error"),_("Argument to --number is not a number but phonebook cache is not present."));
     } else {
       cache = pbook_file_read(filename);
       entry_name = convert_from_system(number);
@@ -367,10 +367,11 @@ char* sms_pdu_create (char* file, char* text, char* number,
 	  found[k] = NULL;
 	}
       }
+      mem_realloc(entry_name,0);
       if (found[0] == NULL) {
-	errexit("Error: no cache entry for \"%s\" found.\n",number);
+	errexit(_("Error: no cache entry for \"%s\" found.\n"),number);
       } else if (found[1] != NULL) {
-	fprintf(stderr,"Error: no unique hit in cache for search term \"%s\":\n",
+	fprintf(stderr,_("Error: no unique hit in cache for search term \"%s\":\n"),
 		(number == NULL) ? "" : number);
 	for (k = 0; found[k] != NULL; ++k) {
 	  entry_number = convert_to_system(found[k]->text,REPMODE_QUESTIONMARK);
@@ -393,10 +394,10 @@ char* sms_pdu_create (char* file, char* text, char* number,
     entry_number = str_dup(number);
   }
 
-  myprintf(0,"%s","Creating PDU...\n");
+  myprintf(0,"%s\n",_("Creating PDU..."));
   pdu = sms_pdu_create_submit(smsinput,entry_number,mysmsopts);
-  myprintf(1,"PDU: %s\n",pdu);
   mem_realloc(entry_number,0);
+  myprintf(1,"PDU: %s\n",pdu);
   return pdu;
 }
 
@@ -415,6 +416,8 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
   int type = 0;
   enum return_code status;
   int listval = -1;
+  char* mark1 = NULL;
+  char* mark2 = NULL;
 
   if (slot >= SCMXX_SLOT_SMS_MIN) {
     if (mark_read) {
@@ -451,31 +454,35 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
   case AT_RET_OTHER:
     break;
   case AT_RET_ERROR:
-    errexit("%s","\nError: unknown cause\n");
+    errexit("\n%s: %s\n",_("Error"),_("unknown cause"));
   case AT_RET_ERROR_CME:
   case AT_RET_ERROR_CMS:
     if (strcasecmp(temp,"INVALID MEMORY INDEX") == 0) {
-      errexit("%s","Error: This slot is not available.\n");
+      errexit("%s: %s\n",_("Error"),_("This slot is not available."));
     } else {
-      errexit("Error: %s.\n",temp);
+      errexit("%s: %s\n",_("Error"),temp);
     }
     break;
   case AT_RET_OK:
     switch (slot) {
     default:
-    case SCMXX_SLOT_ALL: temp = "" ; break;
-    case SCMXX_SLOT_SMS_SENT: temp = "sent "; break;
-    case SCMXX_SLOT_SMS_UNSENT: temp = "unsent "; break;
-    case SCMXX_SLOT_SMS_READ: temp = "read "; break;
-    case SCMXX_SLOT_SMS_UNREAD: temp = "unread "; break;
+    case SCMXX_SLOT_ALL: temp = NULL ; break;
+    case SCMXX_SLOT_SMS_SENT: temp = _("sent"); break;
+    case SCMXX_SLOT_SMS_UNSENT: temp = _("unsent"); break;
+    case SCMXX_SLOT_SMS_READ: temp = _("read"); break;
+    case SCMXX_SLOT_SMS_UNREAD: temp = _("unread"); break;
+    }
+    if (temp != NULL) {
+      errexit(_("There are no %s short messages on the phone.\n"),temp);
+    } else {
+      errexit("%s",_("There are no short messages on the phone.\n"));
     }
-    errexit("There are no %sshort messages on the phone.\n", temp);
     temp = NULL;
     break;
   }
 
   if (slot < SCMXX_SLOT_SMS_MIN) {
-    myprintf(0,"%s","Looking for SMS of specified type...\n");
+    myprintf(0,"%s\n",_("Looking for SMS of specified type..."));
   }
   do {
     slist[retval]=mem_alloc(sizeof(**slist),0);
@@ -485,7 +492,7 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
     } else {
       slist[retval]->slot = atoi(temp);
       temp = index(temp,',');
-      if (temp == NULL) errexit("%s","Output parsing error.\n");
+      if (temp == NULL) errexit("%s\n",_("Output parsing error."));
       ++temp;
     }
     type = atoi(temp);
@@ -497,19 +504,24 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
     mem_realloc(ausgabe,0);
     ausgabe = at_read_line();
     if (!strcmp(ausgabe,"OK")) {
-      errexit("Slot %d is empty.\n", slist[retval]->slot);
+      errexit(_("Slot %d is empty.\n"), slist[retval]->slot);
     } else {
-      myprintf(0,"%s","Found ");
       switch (type) {
-      case 0: myprintf(0,"%s","incoming, unread SMS "); break;
-      case 1: myprintf(0,"%s","incoming, read SMS "); break;
-      case 2: myprintf(0,"%s","outgoing, unsent SMS "); break;
-      case 3: myprintf(0,"%s","outgoing, sent SMS "); break;
-      default: myprintf(0,"%s","SMS of unknown type "); break;
-      }	    
-      myprintf(0,"in slot %d.\n",slist[retval]->slot);
+      case 0: mark1 = _("incoming"); mark2 = _("unread"); break;
+      case 1: mark1 = _("incoming"); mark2 = _("read"); break;
+      case 2: mark1 = _("outgoing"); mark2 = _("unsent"); break;
+      case 3: mark1 = _("outgoing"); mark2 = _("sent"); break;
+      default: break;
+      }
+      if (mark1 != NULL && mark2 != NULL) {
+	myprintf(0,_("Found %s, %s SMS in slot %d.\n"),
+		 mark1, mark2, slist[retval]->slot);
+      } else {
+	myprintf(0,_("Found SMS of unknown type in slot %d.\n"),
+		 slist[retval]->slot)
+      }
       if (strlen(ausgabe) >= sizeof(slist[retval]->tpdu)) {
-	errexit("%s","Returned PDU exceeds buffer size.\n");
+	errexit("%s\n",_("Returned PDU exceeds buffer size."));
       } else {
 	memset(slist[retval]->tpdu,0,sizeof(slist[retval]->tpdu));
 	strncpy(slist[retval]->tpdu,ausgabe,sizeof(slist[retval]->tpdu));
@@ -537,14 +549,13 @@ void sms_print (FILE* filefd, char* pipe, struct sms* data) {
     if (pipefd != NULL) {
       sms_pdu_print(filefd,data);
       if (pclose(pipefd) == -1) {
-	errexit("%s","\nERROR on closing pipe.\n");
+	fprintf(stderr,"\n");
+	errexit(_("Error on closing pipe \"%s\".\n"),pipe);
       }
     }
   }
 }
 
-
-/* void sms_print_single (char* file, char* pipe, struct sms_slot_data* sms) {} */
 void sms_print_all (char* file, char* pipe,
 		    struct sms_slot_data** slist,
 		    char* sort_mode)
@@ -595,14 +606,14 @@ void sms_print_all (char* file, char* pipe,
 
   //merging
   if (sms_multipart_merge(data) < 0) {
-    myprintf(0,"%s","Warning: Concatenated messages cannot be reassembled.\n");
+    myprintf(0,"%s: %s\n",_("Warning"),_("Concatenated messages cannot be reassembled."));
   }
 
   //sorting
   sms_pdu_sort(data,sort_mode);
 
   //looking up number in cache
-  filename = pbook_file_get_cachename();
+  filename = configfile_get_path (PBOOK_FILE_CACHENAME);
   if (filename != NULL &&
       file_accessible_ro(filename) == 1) {
     cache = pbook_file_read(filename);
@@ -627,14 +638,14 @@ void sms_print_all (char* file, char* pipe,
 	  }
 	  //evaluate if only one match was found
 	  if (found[0] == NULL) {
-	    myprintf(1,"\nNo cache hit for number \"%s\" with match size %d\n",
+	    myprintf(1,_("\nNo cache hit for number \"%s\" with match size %d\n"),
 		     addr->digits,hitlen);
 	  } else if (found[1] == NULL) {
-	    myprintf(1,"\nCache hit for number \"%s\" with match size %d\n",
+	    myprintf(1,_("\nCache hit for number \"%s\" with match size %d\n"),
 		     found[0]->number,hitlen);
 	    addr->text = ucs4dup(found[0]->text);
 	  } else {
-	    myprintf(1,"\nNo unique hit in cache for number \"%s\" with match size %d\n",
+	    myprintf(1,_("\nNo unique hit in cache for number \"%s\" with match size %d\n"),
 		     addr->digits,hitlen);
 	    //try to be more specific on the match
 	    ++hitlen;
@@ -677,25 +688,26 @@ void sms_get_direct (char* file, char* pipe) {
   temp = NULL;
 
   if (str_len(file) == 0 && str_len(pipe) == 0) {
-    errexit("%s","Error: No output method was specified.\n");
+    errexit("%s: %s\n",_("Error"),_("No output method was specified."));
   }
 
-  at_command_send(AT_GEN_SMS_PHASE,"1");
+  at_gen_sms_phase(1);
   ausgabe = at_read_line();
-  status = at_line_type(ausgabe,NULL,NULL);
+  status = at_line_type(ausgabe,AT_GEN_SMS_PHASE,NULL);
   mem_realloc(ausgabe,0);
   if (status == AT_RET_ANSWER || status == AT_RET_OTHER) {
     mem_realloc(at_read_line(),0);
   } else {
-    errexit("%s","Error: Could not set Phase 2+ compatible mode.\n");
+    errexit("%s: %s\n",_("Error"),_("Could not set Phase 2+ compatible mode."));
   }
 
-  at_command_send(AT_GEN_MSG_DIRECT,"1,2,0,0,1");
+  
+  at_gen_msg_direct (1,2,2,1);
   ausgabe = at_read_line();
-  status = at_line_type(ausgabe,NULL,NULL);
+  status = at_line_type(ausgabe,AT_GEN_MSG_DIRECT,NULL);
   mem_realloc(ausgabe,0);
   if (status != AT_RET_OK) {
-    errexit("%s","Error: Could not set direct mode.\n");
+    errexit("%s: %s\n",_("Error"),_("Could not set direct mode."));
   }
 
   if (str_len(file) != 0) {
@@ -712,7 +724,7 @@ void sms_get_direct (char* file, char* pipe) {
       mem_realloc(ausgabe,0);
       ausgabe = tty_read_line(at_check_line_end);
       if (ausgabe == NULL) {
-	errexit("Error on reading from device: %s\n", strerror(errno));
+	errexit(_("Error on reading from device: %s\n"), strerror(errno));
       }
       if ((temp = index(ausgabe,'\r')) != NULL ||
 	  (temp = index(ausgabe,'\n')) != NULL) {	
@@ -720,7 +732,8 @@ void sms_get_direct (char* file, char* pipe) {
       }
     } while (strlen(ausgabe) == 0);
 
-    if (strncmp(ausgabe,"+CMT: ,",7)) {
+    if (at_line_type(ausgabe,"+CMT",NULL) == AT_RET_ANSWER /* SMS-DELIVER */
+	|| at_line_type(ausgabe,"+CDS",NULL) == AT_RET_ANSWER /* SMS-STATUS-REPORT */) {
       mem_realloc(ausgabe,0);
       ausgabe = at_read_line();
       encoded = mem_alloc(sizeof(*encoded),0);
@@ -733,9 +746,9 @@ void sms_get_direct (char* file, char* pipe) {
       //doesn't handle concatenated short messages
       decoded = sms_pdu_decode(encoded->type,encoded);
       if (decoded == NULL || decoded->pdu == NULL) {
-	errexit("%s","Error: Failed to decode PDU\n");
+	errexit("%s: %s\n",_("Error"),_("Failed to decode PDU."));
       }
-      struct_sms_init(&data,encoded,decoded);      
+      struct_sms_init(&data,encoded,decoded);
       sms_print(filefd,pipe,&data);
       struct_sms_delete(&data);
       encoded = NULL;
@@ -747,8 +760,20 @@ void sms_get_direct (char* file, char* pipe) {
       ausgabe = at_read_line();
       status = at_line_type(ausgabe,NULL,NULL);
       if (status != AT_RET_OK) {
-	errexit("%s","SMS ACK should have been ok but was not.\n");
+	errexit("%s\n",_("SMS ACK should have been ok but was not."));
       }
+    } else if (at_line_type(ausgabe,"+CMTI",NULL) == AT_RET_ANSWER
+	       || at_line_type(ausgabe,"+CDSI",NULL) == AT_RET_ANSWER) {
+      //not the message itself but an indication to it (very rare case)
+      myprintf(1,"%s: %s\n",_("Notice"),_("A new message was stored in the phone."));
+    } else if (at_line_type(ausgabe,"+CBM",NULL) == AT_RET_ANSWER) {
+      mem_realloc(ausgabe,0);
+      ausgabe = at_read_line();
+      myprintf(0,"%s:\n%s%s\n\n",_("Notice"),
+	       "A cell broadcast message was received, but decoding is not implemented, yet.\n"
+	       "I would like to implement cell broadcast message decoding but has no examples.\n"
+	       "Please mail this to post@hendrik-sattler.de, including the following PDU:\n",
+	       ausgabe);
     }
   }
 }
