@@ -1,4 +1,3 @@
-
 /***************************************************************************
  *   copyright           : (C) 2002 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -15,6 +14,7 @@
 #include "options.h"
 #include "atcommand.h"
 #include "ttyaccess.h"
+#include "gtincl.h"
 
 #include <locale.h>
 #include <string.h>
@@ -37,6 +37,8 @@ int main (int argc, char** argv) {
   char* model;
         
   struct args_t args;
+  int cfargc = 0;
+  char** cfargv;
   char** files;
 
   //start setup
@@ -49,13 +51,18 @@ int main (int argc, char** argv) {
 
   //setting program's locale
   setlocale(LC_ALL,"");
+  scmxx_gettext_init();
 
-  if (argc==1) {
+  cfargv = configfile_read("config");
+  if (cfargv != NULL) while(cfargv[cfargc] != NULL) ++cfargc;
+  
+  if (argc <= 1 && cfargc <= 1) {
     help(argv[0]);
     exit(EXIT_SUCCESS);
   }
 
   args_init(&args);
+  if (cfargv != NULL) mem_realloc(args_parse(cfargc,cfargv,&args),0);
   files = args_parse(argc,argv,&args);
   use_envvars(&args);
 
@@ -74,6 +81,7 @@ int main (int argc, char** argv) {
   //now open the port
   tty_open(&args.port);
 
+  charset_init(args.system_charset);
   at_init(args.pin);
 
   if (args.reset) {
@@ -81,9 +89,9 @@ int main (int argc, char** argv) {
      * the phone may not respond without it because it thinks that all
      * send commands are still data from a previous run
      */
-    myprintf(0,"%s","Setting reset frame...");
+    myprintf(0,"%s...",_("Sending reset frame"));
     tty_write("\x1a\r\n",3);
-    myprintf(0,"%s","done\n");
+    myprintf(0,"%s\n",_("done"));
     exit(EXIT_SUCCESS);
   }
 
@@ -91,9 +99,9 @@ int main (int argc, char** argv) {
    */
   //testing the device
   if (phone_init()) {
-    myprintf(0,"%s","OK, a modem device is present.\n");
+    myprintf(0,"%s\n",_("OK, a modem device is present."));
   } else {
-    errexit("%s","ERROR, cannot communicate with device.\n");
+    errexit("%s: %s\n",_("Error"),_("cannot communicate with device."));
   }
   //disabling command echo
   command_echo(0);
@@ -112,26 +120,29 @@ int main (int argc, char** argv) {
     model = get_model();
     if (vendor!=NULL) {
       if (!strcasecmp(vendor,"SIEMENS")) {
-	myprintf(0,"Detected %s %s\n",vendor,model);
+	myprintf(0,_("Detected %s %s\n"),vendor,model);
 	for (i=0; supported_phones[i] != NULL; ++i) {
-	  myprintf(3,"Comparing to registered model %s\n",supported_phones[i]);
+	  myprintf(3,_("Comparing to registered model %s\n"),supported_phones[i]);
 	  if (!strcmp(model,supported_phones[i])) {
 	    break;
 	  }
 	}
 	if (supported_phones[i] == NULL) {
-	  myprintf(1,"%s","WARNING: this phone was not confirmed to be working with this software.\n");
+	  myprintf(1,"%s: %s\n",_("Warning"),
+		   _("this phone was not confirmed to be working with this software."));
 	}
-      } else {
-	myprintf(0,"%s","WARNING: Phones from this vendor were not confirmed to be working with this software!\n");
+     } else {
+	myprintf(0,"%s: %s\n",_("Warning"),
+		 _("phones from this vendor were not confirmed to be working with this software!"));
       }
     }
     mem_realloc(vendor,0);
     mem_realloc(model,0);
   }
 
-  if (args.time || args.smsc ||
-      args.info || args.meminfo) {
+  if (args.time || args.smsc || args.dial ||
+      args.info || args.meminfo || args.lockinfo ||
+      args.lock != NULL) {
     if (args.time) {
       set_time();
     }
@@ -139,18 +150,28 @@ int main (int argc, char** argv) {
       if (str_len(args.myparams.number)) {
 	set_smsc(args.myparams.number);
       } else {
-	errexit("%s","You must define a number with the --number option.\n");
+	errexit("%s: %s\n",_("Error"),_("you must define a number with the --number option."));
       }
     }
-    if (args.info || args.meminfo) {
-      info(args.myFILE,args.info,args.meminfo);
+    if (args.lock != NULL) {
+      lock_toggle(args.lock,args.pin);
+    }
+    if (args.info || args.meminfo || args.lockinfo) {
+      info(args.myFILE,args.info,args.meminfo,args.lockinfo);
+    }
+    if (args.dial) {
+      if (str_len(args.myparams.number)) {
+	keypad_dial_number(args.myparams.number);
+      } else {
+	errexit("%s: %s\n",_("Error"),_("you must define a number with the --number option."));
+      }
     }
     exit(EXIT_SUCCESS);
   }
 
   switch(args.scmxx_ftype) {
   default:
-    errexit("%s","You must specifiy a valid memory type.\n");
+    errexit("%s: %s\n",_("Error"),_("you must specifiy a valid memory type."));
     break;
   case SCMXX_FTYPE_BINARY:
     file_transfer_manage(args.scmxx_action,files,
