@@ -22,6 +22,7 @@
 #include <timeincl.h>
 #include "smscoding.h"
 #include "smsudh.h"
+#include <gtincl.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -311,7 +312,8 @@ unsigned int sms_pdu_address_decode (char* pdu, struct sms_number* n)
     switch (n->type&SMS_NUMBER_TYPEMASK) {
     case SMS_NUMTYPE_TEXT:
       //those are octets, not semi-octets
-      n->text = sms_data_gsm_decode(pdu+4,(len*4)/7,0);
+      n->text = sms_data_decode(SMS_CHARSET_GSM,
+				pdu+4,(len*4)/7,0);
       break;
     default:
       for (i=0;i<len;i+=2) {
@@ -622,17 +624,10 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
     decoded->pdu = sms_pdu_decode_statusreport(pdu);
     break;
   case SMS_TYPE_SUBMIT_REPORT:
-    fprintf(stderr,"Unsupported pdu type: %s\n","SMS-SUBMIT-REPORT");
-    struct_sms_tpdu_data_delete(decoded);
-    decoded = mem_realloc(decoded,0);
-    break;
   case SMS_TYPE_DELIVER_REPORT:
-    fprintf(stderr,"Unsupported pdu type: %s\n","SMS-DELIVER-REPORT");
-    struct_sms_tpdu_data_delete(decoded);
-    decoded = mem_realloc(decoded,0);
-    break;
   case SMS_TYPE_COMMAND:
-    fprintf(stderr,"Unsupported pdu type: %s\n","SMS-COMMAND");
+    fprintf(stderr,"%s: %s\n",_("Unsupported pdu type"),
+            sms_pdu_print_get_type_string(get_sms_pdu_type(type,hexstr2int(pdu,2))));
     struct_sms_tpdu_data_delete(decoded);
     decoded = mem_realloc(decoded,0);
     break;
