@@ -16,6 +16,20 @@
 
 void tty_open (struct port_args_t* args);
 void tty_close();
+
+#ifdef OS2
+#define speed_t int
+#undef B9600
+#undef B19200
+#undef B38400
+#undef B57600
+#undef B115200
+#define B9600 9600
+#define B19200 19200
+#define B38400 38400
+#define B57600 57600
+#define B115200 115200
+#endif
 speed_t tty_speed(char *newttyspeed);
 
 int  tty_write (const char* data, size_t count);
